/* $Id: uplink.h 411 2010-05-06 19:29:33Z oh2gve $
 *
 * Copyright 2007 Antti Seppänen
 * Copyright 2007, 2008, 2009, 2010 Tapio Aaltonen
 *
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef UPLINK_H
#define UPLINK_H

// Own stuff.
#include "connection.h"
#include "cfg.h"

// std stuff.
#include <vector>
#include <ctime>

// wx stuff.
#include <wx/string.h>
#include <wx/socket.h>


///Implements Connection for TCP client.
class Uplink : public Connection
{
 public:
	
	/// Constructor.
	Uplink(Cfg::UplinkCfg const& cfg, std::string const& login);
	
	/// Destructor.
	virtual ~Uplink();

	// Entry point of our thread. The actual "main" function for this  thread.
	virtual void* Entry();

 private:

	/// Create socket and connect to next server in list.
	/**
	 * If socket already exists, it is recreated.
	 */
	bool connect();
	
	/// Check that socket is connected.
	bool socketIsConnected();

	/// Try to reconnect until success or shutdown request.
	bool reconnect();
	
	/// Print a log line about current socket status.
	/**
	 * Should be called on fatal error to log some info about the error before
	 * giving up.
	 */
	void logError() const;

	
	/// The socket itself.
	wxSocketClient* socket_;

	
	/// Our config
	Cfg::UplinkCfg cfg_;
 
	/// Login string to be sent to server.
	std::string const login_;

	/// Own callsign.
	std::string owncall_;
	
	
	// State variables
	
	/// address:port of current server for logging purposes.
	wxString currentServer_;
	
	/// Index current server.
	unsigned int currentServerNo_;
	
	/// Time of last incoming traffic.
	time_t lastTrafficTime_;
	
	/// Time of last connection attempt.
	time_t lastTryTime_;
	
	
	/// Buffer for data from socket.
	std::string inputBuffer_;
};

#endif // UPLINK_H
